/*
  Interferenz von Schallwellen

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/


p5.disableFriendlyErrors = true; // disables FES

// LAYOUT
let mn;
let button_width;
let button_height;
let control_y;
let text_size;
let line_width;
let rect_round;

let screen;

let resolution_slider;
let volume_slider;

const c_bg = 240;


// GLOBAL
let mic;
let new_level;
let levels = [];
const levels_length = 500;
let time;

let resolution;
let volume;

const max_value_resolution_slider = 500;
const max_value_volume_slider = 5;

let pause;
let audio_started;




function setup() {
  createCanvas(windowWidth, windowHeight);
	screen = createGraphics(width, height);
	screen.clear();

  mic = new p5.AudioIn();
	mic.start();
	for(let i = 0; i < levels_length; i++)
		levels.splice(0, 0, 0);
	time = millis();


  resolution = 250;
	volume = 1;
	threshold = 1;
	
	pause = true;
	audio_started = false;
	
  calculateLayout();

	screen.clear();
	screen.fill(255);	
	screen.noStroke();
	screen.rect(mn, mn, width - mn * 2, control_y - mn, rect_round);

  frameRate(30);
	
	textStyle(BOLD);
}



function draw() {
  background(c_bg);
	
	renderScreen();
  renderLayout();
	resolution_slider.render();
	volume_slider.render();
}



function calculateLayout() {
	height > width ? mn = height * 0.012 : mn = width * 0.01;

	line_width = mn * 0.3;
	control_y = height * 0.7;
  control_size = (height - control_y - mn * 5) / 4;
	screen_middle = control_y * 0.5
	rect_round = mn * 5;

  stroke(0);
  strokeWeight(line_width);
  strokeCap(ROUND);

  fill(255);

	height > width ? text_size = height * 0.04 : text_size = width * 0.03;
  textSize(text_size);
  textAlign(CENTER, TOP);
  textFont('Courier')
  textStyle(NORMAL);
	
	let temp_width = width - mn * 2;
	
	resolution_slider = {
  	pressed : false,
		x : mn,
		y : control_y + mn * 2 + control_size,
		x_offset : control_size * 0.5,
		width : temp_width,
		height : control_size,
		intervall : (temp_width - control_size) / (max_value_resolution_slider - 1),
		value : resolution,
		
		render : function() {
			push();
			rectMode(CORNER);
			fill(255);
			stroke(0);
    	this.pressed ? strokeWeight(line_width * 2) : strokeWeight(line_width); 
      rect(this.x, this.y + this.height * 0.35, this.width, this.height * 0.3, this.height);
      fill(100);
    	ellipse(this.x + (this.value - 1) * this.intervall + this.x_offset, this.y + this.height * 0.5, this.height);
			pop();
		},
		
		updateValue : function() {
			if (this.pressed) {
  			this.value = constrain(round(map(mouseX, this.x + this.x_offset, this.x + this.width - this.x_offset, 1, max_value_resolution_slider)), 1, max_value_resolution_slider);
				resolution = this.value;
			}
		},
		
		checkPressed : function() {
			if (mouseX > this.x && mouseX < this.x + this.width && mouseY > this.y && mouseY < this.y + this.height)
				this.pressed = true;
			else 
				this.pressed = false;
		}
	}
	
	
	volume_slider = {
  	pressed : false,
		x : mn,		
		y : control_y + mn * 4 + control_size * 3,
		x_offset : control_size * 0.5,
		width : temp_width,
		height : control_size,
		intervall : (temp_width - control_size) / (max_value_volume_slider - 1),
		value : volume,
		
		render : function() {
			push();
			rectMode(CORNER);
			fill(255);
			stroke(0);
    	this.pressed ? strokeWeight(line_width * 2) : strokeWeight(line_width);  
      rect(this.x, this.y + this.height * 0.35, this.width, this.height * 0.3, this.height);
      fill(100);
    	ellipse(this.x + (this.value - 1) * this.intervall + this.x_offset, this.y + this.height * 0.5, this.height);
			pop();
		},
		
		updateValue : function() {
			if (this.pressed) {
  			this.value = constrain(map(mouseX, this.x + this.x_offset, this.x + this.width - this.x_offset, 1, max_value_volume_slider), 1, max_value_volume_slider);
				volume = this.value;
			}
		},
		
		checkPressed : function() {
			if (mouseX > this.x && mouseX < this.x + this.width && mouseY > this.y && mouseY < this.y + this.height)
				this.pressed = true;
			else 
				this.pressed = false;
		}
	}
}


function renderLayout() {
  push();
	noFill();
	stroke(0);
	strokeWeight(line_width * 1.4);
	rect(mn, mn, width - mn * 2, control_y - mn, rect_round);
	
	noStroke();
	fill(0);
	text("Auflösung", width * 0.5, control_y + mn * 2);
	
	text("Lautstärke", width * 0.5, control_y + mn * 4 + control_size * 2);
	
	
	if(pause){
		text("- Pause -", width * 0.5, control_size);
		text("zum Fortsetzen klicken", width * 0.5, control_size * 2 + mn);
	}			 
  pop();
}


function renderScreen() {
	if(audio_started) {
		if(!pause) {
			if(millis() - time > resolution) {
				new_level = mic.getLevel();
				levels.splice(0, 0, new_level);
				levels.pop();
				time = millis();
			}
		}


		screen.clear();
		screen.fill(255)
		screen.noStroke();
		screen.rect(mn, mn, width - mn * 2, control_y - mn, rect_round);
		
		screen.noFill();
		screen.stroke(0);
		screen.strokeWeight(line_width);
		
		screen.beginShape();
		for (let i = 0; i < levels.length; i++){
			let x = map(i, 0, levels.length, mn, width - mn);
			let y = map(levels[i] * volume, 0, 1, control_y - mn * 4, mn);
			screen.vertex(x,y);
		}
		screen.endShape();
		screen.fill(c_bg);
		screen.noStroke();
		screen.rect(0, control_y, width, height - control_y);
		screen.rect(0, 0, width, mn);
	}
	
	image(screen, 0, 0);
}


function mPressed() {
	resolution_slider.checkPressed();
	resolution_slider.updateValue();
	volume_slider.checkPressed();
	volume_slider.updateValue();
	
	if(mouseY < control_y) {
		pause ^= true;
	}
	
	if (getAudioContext().state !== 'running') {
    getAudioContext().resume();
		audio_started = true;
  }
}


function mousePressed() {
	mPressed();
	return false;
}


function dragged() {
	resolution_slider.updateValue();
	volume_slider.updateValue();
} 


function touchMoved() {
	dragged();
	
  if(mouseY < height) { 
    return false;
  }
}


function mouseDragged() {
  dragged();
}


function mouseReleased() {
  resolution_slider.pressed = false;
	volume_slider.pressed = false;
}


function windowResized() {
	resizeCanvas(windowWidth, windowHeight);
	if(screen != null)
		screen.remove();
	screen = createGraphics(width, height);
  calculateLayout();
}